declare class PContext {
    x: string;
    pos: number;
    epos: number;
    headpos: number;
    ast: PTree | null;
    state: PState | null;
    memos: PMemo[];
    constructor(inputs: string, pos: number, epos: number);
}
declare class PTree {
    readonly prev: PTree | null;
    readonly tag: string;
    readonly spos: number;
    readonly epos: number;
    readonly child: PTree | null;
    constructor(prev: PTree | null, tag: string, spos: number, epos: number, child: PTree | null);
    isEdge(): boolean;
    dump(inputs: string): string;
}
export declare type PFunc = (px: PContext) => boolean;
export declare const keyRange: (chars: string, ranges: string) => string;
declare type trie = string[] | {
    [key: string]: string[] | string;
};
export declare const match_trie: (px: PContext, d: trie) => boolean;
declare class PMemo {
    key: number;
    pos: number;
    treeState: boolean;
    prev: PTree | null;
    ast: PTree | null;
    result: boolean;
    constructor();
}
declare class PState {
    sid: number;
    val: any;
    prev: PState | null;
    constructor(sid: number, val: any, prev: PState | null);
}
export declare class ParseTree {
    static readonly EMPTY: ParseTree[];
    tag_: string;
    inputs_: string;
    spos_: number;
    epos_: number;
    urn_: string;
    subs_: ParseTree[];
    constructor(tag: string, inputs: string, spos?: number, epos?: number, urn?: string);
    is(tag: string): boolean;
    gettag(): string;
    add(t: ParseTree, edge?: string): void;
    get(key: string): any;
    subNodes(): ParseTree[];
    isSyntaxError(): boolean;
    private pos_;
    beginPosition(): number[];
    endPosition(): number[];
    length(): number;
    keys(): string[];
    toString(): string;
    dump(): string;
    protected strOut(sb: string[]): void;
    showing(msg?: string): string;
}
export declare const quote: (s: string) => string;
export declare type Parser = (inputs: string, options?: any) => ParseTree;
export declare class PAsm {
    static pRule: (peg: {
        [key: string]: PFunc;
    }, name: string, e: PFunc) => void;
    static pEmpty: () => PFunc;
    static pFail: () => PFunc;
    static pAny: () => PFunc;
    static pSkip: () => PFunc;
    static pChar: (text: string) => PFunc;
    static pRange: (chars: string, ranges?: string) => (px: PContext) => boolean;
    static pRef: (generated: any, uname: string) => any;
    static pMemo: (pf: PFunc, mp: number, mpsize: number) => (px: PContext) => boolean;
    static pAnd: (pf: PFunc) => (px: PContext) => boolean;
    static pNot: (pf: PFunc) => (px: PContext) => boolean;
    static pMany: (pf: PFunc) => (px: PContext) => boolean;
    static pMany1: (pf: PFunc) => (px: PContext) => boolean;
    static pOption: (pf: PFunc) => (px: PContext) => boolean;
    static pSeq: (...pfs: PFunc[]) => (px: PContext) => boolean;
    static pSeq2: (pf: PFunc, pf2: PFunc) => (px: PContext) => boolean;
    static pSeq3: (pf: PFunc, pf2: PFunc, pf3: PFunc) => (px: PContext) => boolean;
    static pSeq4: (pf: PFunc, pf2: PFunc, pf3: PFunc, pf4: PFunc) => (px: PContext) => boolean;
    static pOre: (...pfs: PFunc[]) => (px: PContext) => boolean;
    static pOre2: (pf: PFunc, pf2: PFunc) => (px: PContext) => boolean;
    static pOre3: (pf: PFunc, pf2: PFunc, pf3: PFunc) => (px: PContext) => boolean;
    static pOre4: (pf: PFunc, pf2: PFunc, pf3: PFunc, pf4: PFunc) => (px: PContext) => boolean;
    static pDict: (words: string) => (px: PContext) => boolean;
    static pNode: (pf: PFunc, tag: string, shift: number) => (px: PContext) => boolean;
    static pEdge: (edge: string, pf: PFunc) => PFunc;
    static pFold: (edge: string, pf: PFunc, tag: string, shift: number) => (px: PContext) => boolean;
    static pAbs: (pf: PFunc) => (px: PContext) => boolean;
    static pSymbol: (pf: PFunc, sid: number) => (px: PContext) => boolean;
    static pScope: (pf: PFunc) => (px: PContext) => boolean;
    static pExists: (sid: number) => (px: PContext) => boolean;
    static pMatch: (sid: number) => (px: PContext) => boolean;
    static pAndChar: (text: string) => (px: PContext) => boolean;
    static pNotChar: (text: string) => (px: PContext) => boolean;
    static pOptionChar: (text: string) => (px: PContext) => boolean;
    static pManyChar: (text: string) => (px: PContext) => boolean;
    static pMany1Char: (text: string) => (px: PContext) => boolean;
    static pAndRange: (chars: string, ranges?: string) => (px: PContext) => boolean;
    static pNotRange: (chars: string, ranges?: string) => (px: PContext) => boolean;
    static pOptionRange: (chars: string, ranges?: string) => (px: PContext) => boolean;
    static pManyRange: (chars: string, ranges?: string) => (px: PContext) => boolean;
    static pMany1Range: (chars: string, ranges?: string) => (px: PContext) => boolean;
    static pDef: (name: string, pf: PFunc) => (px: PContext) => boolean;
    static pIn: (name: string) => (px: PContext) => boolean;
    static generate: (generated: {
        [key: string]: PFunc;
    }, start: string) => Parser;
    static example: (generated: {
        [key: string]: PFunc;
    }, start: string, input: string) => void;
}
export {};
