"use strict";
// PContext
Object.defineProperty(exports, "__esModule", { value: true });
var PContext = /** @class */ (function () {
    function PContext(inputs, pos, epos) {
        this.x = inputs;
        this.pos = pos;
        this.epos = epos;
        this.headpos = pos;
        this.ast = null;
        this.state = null;
        this.memos = [];
        for (var i = 0; i < 1789; i += 1) {
            this.memos.push(new PMemo());
        }
    }
    return PContext;
}());
var PTree = /** @class */ (function () {
    function PTree(prev, tag, spos, epos, child) {
        this.prev = prev;
        this.tag = tag;
        this.spos = spos;
        this.epos = epos;
        this.child = child;
    }
    PTree.prototype.isEdge = function () {
        return (this.epos < 0);
    };
    PTree.prototype.dump = function (inputs) {
        var sb = [];
        if (this.prev !== null) {
            sb.push(this.prev.dump(inputs));
            sb.push(',');
        }
        sb.push("{#" + this.tag + " ");
        if (this.child === null) {
            sb.push("'");
            sb.push(inputs.substring(this.spos, this.epos));
            sb.push("'");
        }
        else {
            sb.push(this.child.dump(inputs));
        }
        sb.push('}');
        return sb.join('');
    };
    return PTree;
}());
var match_empty = function (px) { return true; };
var pEmpty = function () {
    return match_empty;
};
var match_fail = function (px) { return false; };
var pFail = function () {
    return match_fail;
};
var match_any = function (px) {
    if (px.pos < px.epos) {
        px.pos += 1;
        return true;
    }
    return false;
};
var pAny = function () {
    return match_any;
};
var match_skip = function (px) {
    px.pos = Math.min(px.headpos, px.epos);
    return true;
};
var pSkip = function () {
    return match_skip;
};
/* Char */
var CharCache = {
    '': match_empty
};
var store = function (cache, key, gen) {
    if (!(key in cache)) {
        cache[key] = gen();
    }
    return cache[key];
};
var pChar = function (text) {
    var clen = text.length;
    return store(CharCache, text, function () { return function (px) {
        if (px.x.startsWith(text, px.pos)) {
            px.pos += clen;
            return true;
        }
        return false;
    }; });
};
// const range_min = (chars: string, ranges: string) => {
//   const s = chars + ranges;
//   var min = 0xffff;
//   for (var i = 0; i < s.length; i++) {
//     const c = s.charCodeAt(i)
//     min = Math.min(min, c);
//   }
//   return min;
// }
/* Range */
var range_max = function (chars, ranges) {
    var s = chars + ranges;
    var min = 0;
    for (var i = 0; i < s.length; i++) {
        var c = s.charCodeAt(i);
        min = Math.max(min, c);
    }
    return min;
};
var range_bitmap = function (chars, ranges) {
    var codemax = range_max(chars, ranges) + 1;
    var bitmap = new Uint8Array(((codemax / 8) | 0) + 1);
    bitmap[0] = 2;
    for (var i = 0; i < chars.length; i += 1) {
        var c_1 = chars.charCodeAt(i);
        var n = (c_1 / 8) | 0;
        var mask = 1 << ((c_1 % 8) | 0);
        bitmap[n] |= mask;
    }
    for (var i = 0; i < ranges.length; i += 2) {
        for (var c = ranges.charCodeAt(i); c <= ranges.charCodeAt(i + 1); c += 1) {
            var n = (c / 8) | 0;
            var mask = 1 << ((c % 8) | 0);
            bitmap[n] |= mask;
        }
    }
    return bitmap;
};
var RANGETBL = {
    '\n': '\\n', '\t': '\\t', '\r': '\\r', '\v': '\\v', '\f': '\\f',
    '\\': '\\\\', ']': '\\]', '-': '\\-'
};
exports.keyRange = function (chars, ranges) {
    var sb = [];
    sb.push('[');
    sb.push(translate(chars, RANGETBL));
    var r = ranges;
    for (var i = 0; i < r.length; i += 2) {
        sb.push(translate(r[i], RANGETBL));
        sb.push('-');
        sb.push(translate(r[i + 1], RANGETBL));
    }
    sb.push(']');
    return sb.join('');
};
var Bitmaps = {};
var toBitmap = function (chars, ranges) {
    var key = exports.keyRange(chars, ranges);
    if (!(key in Bitmaps)) {
        Bitmaps[key] = range_bitmap(chars, ranges);
    }
    return Bitmaps[key];
};
var bitmatch = function (c, bitmap) {
    var n = (c / 8) | 0;
    var mask = 1 << ((c % 8) | 0);
    return (n < bitmap.length && (bitmap[n] & mask) === mask);
};
var pRange = function (chars, ranges) {
    if (ranges === void 0) { ranges = ''; }
    var bitmap = toBitmap(chars, ranges);
    return function (px) {
        if (px.pos < px.epos && bitmatch(px.x.charCodeAt(px.pos), bitmap)) {
            px.pos += 1;
            return true;
        }
        return false;
    };
};
/* And */
var pAnd = function (pf) {
    return function (px) {
        var pos = px.pos;
        if (pf(px)) {
            px.headpos = Math.max(px.pos, px.headpos);
            px.pos = pos;
            return true;
        }
        return false;
    };
};
var pNot = function (pf) {
    return function (px) {
        var pos = px.pos;
        var ast = px.ast;
        if (!pf(px)) {
            px.headpos = Math.max(px.pos, px.headpos);
            px.pos = pos;
            px.ast = ast;
            return true;
        }
        return false;
    };
};
var pMany = function (pf) {
    return function (px) {
        var pos = px.pos;
        var ast = px.ast;
        while (pf(px) && pos < px.pos) {
            pos = px.pos;
            ast = px.ast;
        }
        px.headpos = Math.max(px.pos, px.headpos);
        px.pos = pos;
        px.ast = ast;
        return true;
    };
};
var pMany1 = function (pf) {
    return function (px) {
        if (!pf(px)) {
            return false;
        }
        var pos = px.pos;
        var ast = px.ast;
        while (pf(px) && pos < px.pos) {
            pos = px.pos;
            ast = px.ast;
        }
        px.headpos = Math.max(px.pos, px.headpos);
        px.pos = pos;
        px.ast = ast;
        return true;
    };
};
var pOption = function (pf) {
    return function (px) {
        var pos = px.pos;
        var ast = px.ast;
        if (!pf(px)) {
            px.headpos = Math.max(px.pos, px.headpos);
            px.pos = pos;
            px.ast = ast;
        }
        return true;
    };
};
var pSeq2 = function (pf, pf2) {
    return function (px) {
        return pf(px) && pf2(px);
    };
};
var pSeq3 = function (pf, pf2, pf3) {
    return function (px) {
        return pf(px) && pf2(px) && pf3(px);
    };
};
var pSeq4 = function (pf, pf2, pf3, pf4) {
    return function (px) {
        return pf(px) && pf2(px) && pf3(px) && pf4(px);
    };
};
var pSeq = function () {
    var pfs = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        pfs[_i] = arguments[_i];
    }
    return function (px) {
        for (var _i = 0, pfs_1 = pfs; _i < pfs_1.length; _i++) {
            var pf = pfs_1[_i];
            if (!pf(px)) {
                return false;
            }
        }
        return true;
    };
};
/* Ore */
var pOre2 = function (pf, pf2) {
    return function (px) {
        var pos = px.pos;
        var ast = px.ast;
        if (pf(px)) {
            return true;
        }
        px.headpos = Math.max(px.pos, px.headpos);
        px.pos = pos;
        px.ast = ast;
        return pf2(px);
    };
};
var pOre3 = function (pf, pf2, pf3) {
    return function (px) {
        var pos = px.pos;
        var ast = px.ast;
        if (pf(px)) {
            return true;
        }
        px.headpos = Math.max(px.pos, px.headpos);
        px.pos = pos;
        px.ast = ast;
        if (pf2(px)) {
            return true;
        }
        px.headpos = Math.max(px.pos, px.headpos);
        px.pos = pos;
        px.ast = ast;
        return pf3(px);
    };
};
var pOre4 = function (pf, pf2, pf3, pf4) {
    return function (px) {
        var pos = px.pos;
        var ast = px.ast;
        if (pf(px)) {
            return true;
        }
        px.headpos = Math.max(px.pos, px.headpos);
        px.pos = pos;
        px.ast = ast;
        if (pf2(px)) {
            return true;
        }
        px.headpos = Math.max(px.pos, px.headpos);
        px.pos = pos;
        px.ast = ast;
        if (pf3(px)) {
            return true;
        }
        px.headpos = Math.max(px.pos, px.headpos);
        px.pos = pos;
        px.ast = ast;
        return pf4(px);
    };
};
var pOre = function () {
    var pfs = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        pfs[_i] = arguments[_i];
    }
    return function (px) {
        var pos = px.pos;
        var ast = px.ast;
        for (var _i = 0, pfs_2 = pfs; _i < pfs_2.length; _i++) {
            var pf = pfs_2[_i];
            if (pf(px)) {
                return true;
            }
            px.headpos = Math.max(px.pos, px.headpos);
            px.pos = pos;
            px.ast = ast;
        }
        return false;
    };
};
/* Dict */
var make_words = function (ss) {
    var dic = {};
    var sss = [];
    dic[''] = sss;
    for (var _i = 0, ss_1 = ss; _i < ss_1.length; _i++) {
        var w = ss_1[_i];
        sss.push(w);
        if (w.length === 0) {
            return dic;
        }
        var key = w.substring(0, 1);
        if (!(key in dic)) {
            dic[key] = [];
        }
        dic[key].push(w.substring(1));
    }
    return dic;
};
var make_trie = function (ss) {
    var dic = make_words(ss);
    ss = dic[''];
    if (ss.length < 10) {
        return ss;
    }
    delete dic[''];
    for (var _i = 0, _a = Object.keys(dic); _i < _a.length; _i++) {
        var key = _a[_i];
        ss = dic[key];
        if (ss.length === 1) {
            dic[key] = ss[0];
        }
        else {
            dic[key] = make_trie(ss);
        }
    }
    return dic;
};
exports.match_trie = function (px, d) {
    if (Array.isArray(d)) {
        var inputs = px.x;
        var pos = px.pos;
        for (var _i = 0, d_1 = d; _i < d_1.length; _i++) {
            var w = d_1[_i];
            if (inputs.startsWith(w, pos)) {
                px.pos += w.length;
                return true;
            }
        }
        return false;
    }
    else if (px.pos < px.epos) {
        var c = px.x[px.pos++];
        var suffix = d[c];
        if (typeof suffix === 'string') {
            if (px.x.startsWith(suffix, px.pos)) {
                px.pos += (suffix.length);
                return true;
            }
            return false;
        }
        return suffix !== undefined ? exports.match_trie(px, suffix) : false;
    }
    return false;
};
var pDict = function (words) {
    var trie = make_trie(words.split(' '));
    if (Array.isArray(trie)) {
        var ss_2 = trie;
        return function (px) {
            var pos = px.pos;
            for (var _i = 0, ss_3 = ss_2; _i < ss_3.length; _i++) {
                var s = ss_3[_i];
                if (px.x.startsWith(s, pos)) {
                    px.pos += s.length;
                    return true;
                }
            }
            return false;
        };
    }
    else {
        return function (px) { return exports.match_trie(px, trie); };
    }
};
/* Ref */
var pRef = function (generated, uname) {
    if (!(uname in generated)) {
        generated[uname] = function (px) { return generated[uname](px); };
    }
    return generated[uname];
};
var PMemo = /** @class */ (function () {
    function PMemo() {
        this.key = -1;
        this.pos = 0;
        this.ast = null;
        this.prev = null;
        this.result = false;
        this.treeState = false;
    }
    return PMemo;
}());
var pMemo = function (pf, mp, mpsize) {
    var disabled = false;
    var hit = 0;
    var miss = 0;
    return function (px) {
        if (disabled)
            return pf(px);
        var key = (mpsize * px.pos) + mp;
        var m = px.memos[(key % 1789) | 0];
        if (m.key == key) {
            if (m.treeState) {
                if (m.prev === px.ast) {
                    px.pos = m.pos;
                    px.ast = m.ast;
                    hit += 1;
                    return m.result;
                }
            }
            else {
                px.pos = m.pos;
                return m.result;
            }
        }
        var prev = px.ast;
        m.result = pf(px);
        m.pos = px.pos;
        m.key = key;
        if (m.result && prev != px.ast) {
            m.treeState = true;
            m.prev = prev;
            m.ast = px.ast;
        }
        else {
            m.treeState = false;
        }
        miss += 1;
        if (miss % 100 === 0 && (hit / miss) < 0.05) {
            disabled = false;
        }
        return m.result;
    };
};
/* Tree Construction */
var pNode = function (pf, tag, shift) {
    return function (px) {
        var pos = px.pos;
        var prev = px.ast;
        px.ast = null;
        if (pf(px)) {
            px.ast = new PTree(prev, tag, pos + shift, px.pos, px.ast);
            return true;
        }
        return false;
    };
};
var pEdge = function (edge, pf) {
    if (edge === '') {
        return pf;
    }
    return function (px) {
        var pos = px.pos;
        var prev = px.ast;
        px.ast = null;
        if (pf(px)) {
            px.ast = new PTree(prev, edge, pos, -px.pos, px.ast);
            return true;
        }
        return false;
    };
};
var pFold = function (edge, pf, tag, shift) {
    if (edge !== '') {
        return function (px) {
            var pos = px.pos;
            var pt = px.ast;
            var prev = pt ? pt.prev : null;
            pt = pt ? (prev ? new PTree(null, pt.tag, pt.epos, pt.epos, pt.child) : pt) : null;
            px.ast = new PTree(null, edge, pos, -pos, pt);
            if (pf(px)) {
                px.ast = new PTree(prev, tag, pos, px.pos + shift, px.ast);
                return true;
            }
            return false;
        };
    }
    else {
        return function (px) {
            var pos = px.pos;
            var pt = px.ast;
            var prev = (pt !== null) ? pt.prev : null;
            px.ast = pt ? (prev ? new PTree(null, pt.tag, pt.spos, pt.epos, pt.child) : pt) : null;
            if (pf(px)) {
                px.ast = new PTree(prev, tag, pos, px.pos + shift, px.ast);
                return true;
            }
            return false;
        };
    }
};
var pAbs = function (pf) {
    return function (px) {
        var ast = px.ast;
        if (pf(px)) {
            px.ast = ast;
            return true;
        }
        return false;
    };
};
// State 
var PState = /** @class */ (function () {
    function PState(sid, val, prev) {
        this.sid = sid;
        this.val = val;
        this.prev = prev;
    }
    return PState;
}());
var getstate = function (state, sid) {
    while (state !== null) {
        if (state.sid === sid) {
            return state;
        }
        state = state.prev;
    }
    return state;
};
var pSymbol = function (pf, sid) {
    return function (px) {
        var pos = px.pos;
        if (pf(px)) {
            px.state = new PState(sid, px.x.substring(pos, px.pos), px.state);
            return true;
        }
        return false;
    };
};
var pScope = function (pf) {
    return function (px) {
        var state = px.state;
        if (pf(px)) {
            px.state = state;
            return true;
        }
        return false;
    };
};
var pExists = function (sid) {
    return function (px) {
        return getstate(px.state, sid) !== null;
    };
};
var pMatch = function (sid) {
    return function (px) {
        var state = getstate(px.state, sid);
        if (state !== null && px.x.startsWith(state.val, px.pos)) {
            px.pos += state.val.length;
            return true;
        }
        return false;
    };
};
var pDef = function (name, pf) {
    return function (px) {
        var pos = px.pos;
        if (pf(px)) {
            var s = px.x.substring(pos, px.pos);
            if (s.length === 0) {
                return true;
            }
            var ss = px[name] || [];
            ss.push(s);
            ss.sort(function (x, y) { return x.length - y.length; });
            px[name] = ss;
            console.log('@TODO ss');
            return true;
        }
        return false;
    };
};
var pIn = function (name) {
    return function (px) {
        var ss = px[name];
        if (ss) {
            for (var _i = 0, ss_4 = ss; _i < ss_4.length; _i++) {
                var s = ss_4[_i];
                if (px.x.startsWith(s, px.pos)) {
                    px.pos += s.length;
                    return true;
                }
            }
        }
        return false;
    };
};
// Optimized
var pAndChar = function (text) {
    return function (px) {
        return px.x.startsWith(text, px.pos);
    };
};
var pNotChar = function (text) {
    return function (px) {
        return !px.x.startsWith(text, px.pos);
    };
};
var pOptionChar = function (text) {
    var clen = text.length;
    return function (px) {
        if (px.x.startsWith(text, px.pos)) {
            px.pos += clen;
        }
        return true;
    };
};
var pManyChar = function (text) {
    var clen = text.length;
    return function (px) {
        while (px.x.startsWith(text, px.pos)) {
            px.pos += clen;
        }
        return true;
    };
};
var pMany1Char = function (text) {
    var clen = text.length;
    return function (px) {
        if (!px.x.startsWith(text, px.pos)) {
            return false;
        }
        px.pos += clen;
        while (px.x.startsWith(text, px.pos)) {
            px.pos += clen;
        }
        return true;
    };
};
var pAndRange = function (chars, ranges) {
    if (ranges === void 0) { ranges = ''; }
    var bitmap = toBitmap(chars, ranges);
    return function (px) {
        return (px.pos < px.epos && bitmatch(px.x.charCodeAt(px.pos), bitmap));
    };
};
var pNotRange = function (chars, ranges) {
    if (ranges === void 0) { ranges = ''; }
    var bitmap = toBitmap(chars, ranges);
    return function (px) {
        return !(px.pos < px.epos && bitmatch(px.x.charCodeAt(px.pos), bitmap));
    };
};
var pOptionRange = function (chars, ranges) {
    if (ranges === void 0) { ranges = ''; }
    var bitmap = toBitmap(chars, ranges);
    return function (px) {
        if (px.pos < px.epos && bitmatch(px.x.charCodeAt(px.pos), bitmap)) {
            px.pos += 1;
        }
        return true;
    };
};
var pManyRange = function (chars, ranges) {
    if (ranges === void 0) { ranges = ''; }
    var bitmap = toBitmap(chars, ranges);
    return function (px) {
        while (px.pos < px.epos && bitmatch(px.x.charCodeAt(px.pos), bitmap)) {
            px.pos += 1;
        }
        return true;
    };
};
var pMany1Range = function (chars, ranges) {
    if (ranges === void 0) { ranges = ''; }
    var bitmap = toBitmap(chars, ranges);
    return function (px) {
        if (px.pos < px.epos && bitmatch(px.x.charCodeAt(px.pos), bitmap)) {
            px.pos += 1;
            while (px.pos < px.epos && bitmatch(px.x.charCodeAt(px.pos), bitmap)) {
                px.pos += 1;
            }
            return true;
        }
        return false;
    };
};
// ParseTree
var ParseTree = /** @class */ (function () {
    function ParseTree(tag, inputs, spos, epos, urn) {
        if (spos === void 0) { spos = 0; }
        if (epos === void 0) { epos = -1; }
        if (urn === void 0) { urn = '(unknown source)'; }
        this.tag_ = tag;
        this.inputs_ = inputs;
        this.spos_ = spos;
        this.epos_ = (epos === -1) ? (inputs.length - spos) : epos;
        this.urn_ = urn;
        this.subs_ = ParseTree.EMPTY;
    }
    ParseTree.prototype.is = function (tag) {
        return this.tag_ === tag;
    };
    ParseTree.prototype.gettag = function () {
        return this.tag_;
    };
    ParseTree.prototype.add = function (t, edge) {
        if (edge === void 0) { edge = ''; }
        if (edge === '') {
            if (this.subs_ === ParseTree.EMPTY) {
                this.subs_ = [];
            }
            this.subs_.push(t);
        }
        else {
            this[edge] = t;
        }
    };
    ParseTree.prototype.get = function (key) {
        return this[key];
    };
    ParseTree.prototype.subNodes = function () {
        return this.subs_;
    };
    ParseTree.prototype.isSyntaxError = function () {
        return this.tag_ === 'err';
    };
    ParseTree.prototype.pos_ = function (pos) {
        var s = this.inputs_;
        pos = Math.min(pos, s.length);
        var row = 0;
        var col = 0;
        for (var i = 0; i <= pos; i += 1) {
            if (s.charCodeAt(i) == 10) {
                row += 1;
                col = 0;
            }
            else {
                col += 1;
            }
        }
        return [pos, row, col];
    };
    ParseTree.prototype.beginPosition = function () {
        return this.pos_(this.spos_);
    };
    ParseTree.prototype.endPosition = function () {
        return this.pos_(this.spos_);
    };
    ParseTree.prototype.length = function () {
        return this.epos_ - this.spos_;
    };
    ParseTree.prototype.keys = function () {
        return Object.keys(this).filter(function (x) { return !x.endsWith('_'); });
    };
    ParseTree.prototype.toString = function () {
        return this.inputs_.substring(this.spos_, this.epos_);
    };
    ParseTree.prototype.dump = function () {
        var sb = [];
        this.strOut(sb);
        return sb.join('');
    };
    ParseTree.prototype.strOut = function (sb) {
        var c = 0;
        sb.push("[#");
        sb.push(this.tag_);
        for (var _i = 0, _a = this.subNodes(); _i < _a.length; _i++) {
            var node = _a[_i];
            c += 1;
            sb.push(" ");
            node.strOut(sb);
        }
        for (var _b = 0, _c = this.keys(); _b < _c.length; _b++) {
            var key = _c[_b];
            sb.push(" " + key + " = ");
            this[key].strOut(sb);
        }
        if (c == 0) {
            sb.push(' ');
            sb.push(exports.quote(this.inputs_.substring(this.spos_, this.epos_)));
        }
        sb.push("]");
    };
    ParseTree.prototype.showing = function (msg) {
        if (msg === void 0) { msg = 'Syntax Error'; }
        var p = this.beginPosition();
        var pos = p[0];
        var row = p[1];
        var col = p[2];
        return "(" + this.urn_ + ":" + row + "+" + col + ") " + msg;
    };
    ParseTree.EMPTY = [];
    return ParseTree;
}());
exports.ParseTree = ParseTree;
var PTree2ParseTree = function (pt, urn, inputs) {
    if (pt.prev !== null) {
        return PTree2ParseTreeChild('', urn, inputs, pt.spos, pt.epos, pt);
    }
    else {
        return PTree2ParseTreeChild(pt.tag, urn, inputs, pt.spos, pt.epos, pt.child);
    }
};
var PTree2ParseTreeChild = function (tag, urn, inputs, spos, epos, sub) {
    var t = new ParseTree(tag, inputs, spos, epos, urn);
    while (sub !== null) {
        if (sub.isEdge()) {
            if (sub.child === null) {
                var tt = PTree2ParseTreeChild('', urn, inputs, sub.spos, Math.abs(sub.epos), null);
            }
            else {
                tt = PTree2ParseTree(sub.child, urn, inputs);
            }
            t.add(tt, sub.tag);
        }
        else {
            t.add(PTree2ParseTreeChild(sub.tag, urn, inputs, sub.spos, sub.epos, sub.child));
        }
        sub = sub.prev;
    }
    var tail = t.subs_.length - 1;
    for (var i = 0; i < (tail + 1) / 2; i += 1) {
        var t0 = t.subs_[i];
        t.subs_[i] = t.subs_[tail - i];
        t.subs_[tail - i] = t0;
    }
    return t;
};
var translate = function (s, dic) {
    var foundESC = false;
    for (var _i = 0, _a = Object.keys(dic); _i < _a.length; _i++) {
        var c = _a[_i];
        if (s.indexOf(c) !== -1) {
            foundESC = true;
            break;
        }
    }
    if (foundESC) {
        var sb = [];
        for (var _b = 0, s_1 = s; _b < s_1.length; _b++) {
            var c = s_1[_b];
            if (c in dic) {
                sb.push(dic[c]);
            }
            else {
                sb.push(c);
            }
        }
        return sb.join('');
    }
    return s;
};
var ESCTBL = { '\n': '\\n', '\t': '\\t', '\r': '\\r', '\v': '\\v', '\f': '\\f', '\\': '\\\\', "'": "\\'" };
exports.quote = function (s) {
    return "'" + translate(s, ESCTBL) + "'";
};
var PAsm = /** @class */ (function () {
    function PAsm() {
    }
    PAsm.pRule = function (peg, name, e) {
        peg[name] = e;
    };
    PAsm.pEmpty = pEmpty;
    PAsm.pFail = pFail;
    PAsm.pAny = pAny;
    PAsm.pSkip = pSkip;
    PAsm.pChar = pChar;
    PAsm.pRange = pRange;
    PAsm.pRef = pRef;
    PAsm.pMemo = pMemo;
    PAsm.pAnd = pAnd;
    PAsm.pNot = pNot;
    PAsm.pMany = pMany;
    PAsm.pMany1 = pMany1;
    PAsm.pOption = pOption;
    PAsm.pSeq = pSeq;
    PAsm.pSeq2 = pSeq2;
    PAsm.pSeq3 = pSeq3;
    PAsm.pSeq4 = pSeq4;
    PAsm.pOre = pOre;
    PAsm.pOre2 = pOre2;
    PAsm.pOre3 = pOre3;
    PAsm.pOre4 = pOre4;
    PAsm.pDict = pDict;
    PAsm.pNode = pNode;
    PAsm.pEdge = pEdge;
    PAsm.pFold = pFold;
    PAsm.pAbs = pAbs;
    /* Symbol */
    PAsm.pSymbol = pSymbol;
    PAsm.pScope = pScope;
    PAsm.pExists = pExists;
    PAsm.pMatch = pMatch;
    /* Optimize */
    PAsm.pAndChar = pAndChar;
    PAsm.pNotChar = pNotChar;
    PAsm.pOptionChar = pOptionChar;
    PAsm.pManyChar = pManyChar;
    PAsm.pMany1Char = pMany1Char;
    PAsm.pAndRange = pAndRange;
    PAsm.pNotRange = pNotRange;
    PAsm.pOptionRange = pOptionRange;
    PAsm.pManyRange = pManyRange;
    PAsm.pMany1Range = pMany1Range;
    PAsm.pDef = pDef;
    PAsm.pIn = pIn;
    PAsm.generate = function (generated, start) {
        var pf = generated[start];
        return function (inputs, options) {
            options = options || {};
            var pos = options.pos || options.spos || 0;
            var epos = options.epos || (inputs.length - pos);
            var px = new PContext(inputs, pos, epos);
            if (pf(px)) {
                if (!px.ast) {
                    px.ast = new PTree(null, "", pos, px.pos, null);
                }
            }
            else {
                px.ast = new PTree(null, "err", px.headpos, px.headpos, null);
            }
            var conv = options.conv || PTree2ParseTree;
            var urn = options.urn || '(unknown source)';
            return conv(px.ast, urn, inputs);
        };
    };
    PAsm.example = function (generated, start, input) {
        var p = PAsm.generate(generated, start);
        var t = p(input);
        console.log(t.dump());
    };
    return PAsm;
}());
exports.PAsm = PAsm;
//# sourceMappingURL=pasm.js.map